package rocks.zipcode.atm;

import com.sun.javafx.binding.StringFormatter;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import rocks.zipcode.atm.bank.AccountData;
import rocks.zipcode.atm.bank.Bank;

import java.util.ArrayList;
import java.util.Observable;
import java.util.function.Consumer;
import java.util.function.Supplier;

/**
 * @author ZipCodeWilmington
 */
public class CashMachine {

    private final Bank bank;
    private AccountData accountData = null;

    public CashMachine(Bank bank) {

        this.bank = bank;


    }

    public int getId() {
        return accountData.getId();
    }

    public String getName() {
        return accountData.getName();
    }

    public String getEmail() {
        return accountData.getEmail();
    }


    public Float getBalance() {

        return accountData.getBalance();
    }

    private Consumer<AccountData> update = data -> {
        accountData = data;
    };

    public void resetPassword(int accountNumber, String newPassword) {
        bank.resetPassword(accountNumber, newPassword);
    }

    public ObservableList<Integer> getIdList() {

        ObservableList<Integer> idList = FXCollections.observableArrayList(bank.getIdList());

        return idList;
    }

    public String message(String message) {
        return message;
    }

    public boolean checkPassword(int accountNumber, String password) {

        password.toLowerCase();
        return bank.getPassword(accountNumber).equals(password);
    }

    public void login(int id) {
        tryCall(
                () -> bank.getAccountById(id),
                update
        );
    }

    public void deposit(float amount) {
        if (accountData != null) {
            tryCall(
                    () -> bank.deposit(accountData, amount),
                    update
            );
        }
    }

    public void withdraw(float amount) {
        if (accountData != null) {
            tryCall(
                    () -> bank.withdraw(accountData, amount),
                    update
            );

        }
    }


    public void exit() {
        if (accountData != null) {
            accountData = null;
        }
    }

    @Override
    public String toString() {
        return accountData != null ? accountData.toString() : "Incorrect account number. \nTry again or click 'Forgot Account Number'";
    }


    private <T> void tryCall(Supplier<ActionResult<T> > action, Consumer<T> postAction) {
        try {
            ActionResult<T> result = action.get();
            if (result.isSuccess()) {
                T data = result.getData();
                postAction.accept(data);
            } else {
                String errorMessage = result.getErrorMessage();
                throw new RuntimeException(errorMessage);
            }
        } catch (Exception e) {
            System.out.println("Error: " + e.getMessage());
        }
    }

}
